from network import LoRa
import socket
import time
import ubinascii
import pycom
from pysense import Pycoproc
import cayenneLPP
# https://github.com/jojo-/py-cayenne-lpp/
from machine import Timer
from L76GNSS import L76GNSS

print('Pytrack-TTN-GPS')
print('LoPy4')
pycom.heartbeat(False)

gnss = L76GNSS(timeout=30)
pc = Pycoproc()

# Initialise LoRa in LORAWAN mode.
# Please pick the region that matches where you are using the device:
# Asia = LoRa.AS923
# Australia = LoRa.AU915
# Europe = LoRa.EU868
# United States = LoRa.US915
lora = LoRa(mode=LoRa.LORAWAN, region=LoRa.US915)

for index in range(0, 8):
    lora.remove_channel(index)

for index in range(16, 65):
    lora.remove_channel(index)

for index in range(66, 72):
    lora.remove_channel(index)

# create an OTAA authentication parameters
app_eui = ubinascii.unhexlify('70B3D57ED001FACD')
app_key = ubinascii.unhexlify('EF37EB4214BF4857D048AE79D83B149C')

# join a network using OTAA (Over the Air Activation)
lora.join(activation=LoRa.OTAA, auth=(app_eui, app_key), timeout=0)

# wait until the module has joined the network

while not lora.has_joined():
    time.sleep(2.5)
    pycom.rgbled(0xFFFF00) # yellow
    time.sleep(0.2)
    pycom.rgbled(0x000000) # led off
    print('Not yet joined...')
pycom.heartbeat(True)

# create a LoRa socket
s = socket.socket(socket.AF_LORA, socket.SOCK_RAW)

# set the LoRaWAN data rate
# list of data rates: https://blog.dbrgn.ch/2017/6/23/lorawan-data-rates/
s.setsockopt(socket.SOL_LORA, socket.SO_DR, 1)


class Sender:

    def __init__(self):
        print('starting timer...')
        # send an immediate data payload
        self.__alarm = Timer.Alarm(self._send_handler, 1, periodic=False)
        # send a periodic data payload
        self.__alarm = Timer.Alarm(self._send_handler, 15, periodic=True)
        self.counter = 0

    def _send_handler(self, alarm):
        print('sending...')
        # creating Cayenne LPP packet
        lpp = cayenneLPP.CayenneLPP(size = 30, sock = s)

        # take measurements
        coords = gnss.coordinates()
        battery = pc.read_battery_voltage()
        self.counter = self.counter + 1

        # send measurements
        print('battery', battery)
        print('counter', self.counter)
        print('lat: {}, long: {}'.format(coords[0], coords[1]))

        lpp.add_analog_input(battery, channel = 105)
        lpp.add_analog_input(self.counter, channel = 106)
        if (coords[0] and coords[1]):
            lpp.add_gps(coords[0], coords[1],-1, channel = 12)
        print('payload size: {}'.format(lpp.get_size()))
        # sending the packet via the socket
        try:
            lpp.send()
        except OSError as e:
            print(e)
            pycom.heartbeat(False)
            for x in range(10):
                pycom.rgbled(0xFF0000) # red
                time.sleep(0.2)
                pycom.rgbled(0x000000) # off
                time.sleep(0.2)
            import machine
            machine.reset()
        pycom.heartbeat(False)
        pycom.rgbled(0x00FF00) # green
        time.sleep(0.2)
        if (not coords[0] and not coords[1]):
             pycom.heartbeat(False)
             pycom.rgbled(0xFFFF00) # yellow
             time.sleep(0.2)
        pycom.heartbeat(True)
        if pc.button_pressed():
            self.__alarm.cancel()
            pycom.heartbeat(False)
            pycom.rgbled(0xFF0000) # red
            time.sleep(5)
            pycom.rgbled(0x000000) # off

sender = Sender()
